'\"
'\" Copyright (c) 1996, Scotter <scotter@iname.com>
'\"
'\" 
.TH tclbin n 0.5.2 Pui "Pilot-Unix Interface"
.BS
.SH NAME
tclpui \- Tcl front-end to pilot-unix interface database
.SH SYNOPSIS
\fBload tclpui.so\fR
.BE

.SH DESCRIPTION
.PP
Provides a Tcl front-end to the Pilot-Unix Interface (PUI) database
format. Several commands are created as described below:
.TP
\fBpuiNew \fIfilename databasename\fR
Create a new database called \fIfilename\fR. The \fIdatabasename\fR is
the name of the database type. The four types known to tclpui are:
AddressDB, MemoDB, ToDoDB, and DatebookDB.
If the routine
succeeds, it will return an identifier to be used with
all subsequent database operations. If it fails it will return 0.
.TP
\fBpuiOpen \fIfilename \fR?\fImode\fR?
Open existing database \fIfilename\fR. If \fImode\fR is unspecified or "r",
the database will be opened read-only. If \fImode\fR is "w", the
file is opened for writing.
If the routine succeeds, it will return an identifier to be used with
all subsequent database operations. If it fails it will return 0.
.TP
\fBpuiClose \fIdb\fR
Close the database specified by identifier \fIdb\fR. 
The identifier should be the result of a prior \fBpuiNew\fR or \fBpuiOpen\fR.
.TP
\fBpuiName \fIdb\fR
Return the database name of the database specified by the identifier \fIdb\fR. 
.TP
\fBpuiVersion \fIdb\fR
Return the database file version string for the database specified
by the identifier \fIdb\fR.
.PP
Each record of the database has a unique \fIid\fR associated with it.
Id's are used when reading and writing entries from/to the database.
The following routines provide access to these \fIid\fR's.
.TP
\fBpuiFirst \fIdb\fR
Return the first id from the database specified by \fIdb\fR.
.TP
\fBpuiLast \fIdb\fR
Return the last id from the database specified by \fIdb\fR.
.TP
\fBpuiPrev \fIdb\fR \fIid\fR
Return the id for the entry in the database \fIdb\fR prior to record \fIid\fR.
.TP
\fBpuiNext \fIdb\fR \fIid\fR
Return the id for the entry in the database \fIdb\fR following record \fIid\fR.
.TP
\fBpuiList \fIdb\fR
Return a list of all the NON-DELETED id's in database \fIdb\fR.
The other iterators give access to all ids.
.PP
Data for fields in a record is transferred via array elements.
When reading and writing records from the database, the name
of the array variable name is passed to the reading or writing routine.
All database types will create \fIcategory\fR and \fIattribute\fR
array elements. Beyond these, the elements will depend on specific
database type. After reading a record into a variable array,
\fBarray names\fB \fIvarName\fR can be used to list all the elements.
Elements which represent dates and times are converted into UNIX-style
seconds, appropriate for use with the tcl \fBclock\fB command.
.TP
\fBpuiRead \fIdb\fR \fIid|DBInfo|AppInfo\fR \fIvarName\fR
Read record \fIid\fR from database \fIdb\fR and place the fields
in the array \fIvarName\fR. As two special cases, the id can
be replaced with either the tag "DBInfo" or "AppInfo" to
read fields from those records.
.TP
\fBpuiWrite \fIdb\fR \fIid|DBInfo|AppInfo\fR \fIvarName\fR
Write record \fIid\fR to database \fIdb\fR from the fields
in the array \fIvarName\fR. If \fIid\fR exists, the existing
record is replaced with the new contents. Otherwise a new record
is created and appended to the list. If \fIid\fR is 0, a new record
is created with a unique id. \fBpuiWrite\fR returns the \fIid\fR
which in the latter case will be new. As two special cases, the
id can be replaced with either the tag "DBInfo" or "AppInfo" to
write fields from those records.
.TP
\fBpuiDelete \fIdb\fR \fIid\fR ?\fIarchive\fR?
Mark the given record for deletion. This does not actually delete
the record, as we need to keep the tagged record around until
we can propagate the change to the pilot. (Deleted records will
be flushed during the next synchronization.) If the optional \fIarchive\fR
flag is set, the record will also be marked to be archived.
.TP
\fBMemoDB\fR
.RS
.IP \fBcategory\fR 10
The category field of the memo. (integer)
.IP \fBattribute\fR 10
The attribute field for the memo. (integer)
.IP \fBtext\fR 10
The text of the memo. (string)
.RE
.TP
\fBToDoDB\fR
.RS
.IP \fBcategory\fR 10
The category field of the todo. (integer)
.IP \fBattribute\fR 10
The attribute field for the todo. (integer)
.IP \fBindefinite\fR 10
Set true if the todo item has no due date (integer)
.IP \fBdue\fR 10
The due date of a non-indefinite todo (\fBclock\fR format time)
.IP \fBpriority\fR 10
The priority field of the todo (integer)
.IP \fBcomplete\fR 10
The item-complete flag of the todo (integer 0|1)
.IP \fBdescription\fR 10
The text description of the todo (string)
.IP \fBnote\fR 10
An optional note for the todo (string)
.RE
.TP
\fBAddressDB\fR
.RS
.IP \fBcategory\fR 10
The category field of the address. (integer)
.IP \fBattribute\fR 10
The attribute field for the address. (integer)
.IP \fBlastname\fR 10
(string)
.IP \fBfirstname\fR 10
(string)
.IP \fBcompany\fR 10
(string)
.IP \fBphone1\fR 10
(string)
.IP \fBphone2\fR 10
(string)
.IP \fBphone3\fR 10
(string)
.IP \fBphone4\fR 10
(string)
.IP \fBphone5\fR 10
(string)
.IP \fBaddress\fR 10
(string)
.IP \fBcity\fR 10
(string)
.IP \fBstate\fR 10
(string)
.IP \fBzip\fR 10
(string)
.IP \fBcountry\fR 10
(string)
.IP \fBtitle\fR 10
(string)
.IP \fBcustom1\fR 10
(string)
.IP \fBcustom2\fR 10
(string)
.IP \fBcustom3\fR 10
(string)
.IP \fBcustom4\fR 10
(string)
.IP \fBnote\fR 10
(string)
.IP \fBphonelabel1\fR 10
Label for phone entry 1
(integer 0-7 indexing Work Home Fax Other E-mail Main Pager Mobile)
.IP \fBphonelabel2\fR 10
Label for phone entry 2
(integer 0-7 indexing Work Home Fax Other E-mail Main Pager Mobile)
.IP \fBphonelabel3\fR 10
Label for phone entry 3
(integer 0-7 indexing Work Home Fax Other E-mail Main Pager Mobile)
.IP \fBphonelabel4\fR 10
Label for phone entry 4
(integer 0-7 indexing Work Home Fax Other E-mail Main Pager Mobile)
.IP \fBphonelabel5\fR 10
Label for phone entry 5
(integer 0-7 indexing Work Home Fax Other E-mail Main Pager Mobile)
.IP \fBwhichphone\fR 10
Which phone entry to display in "short list" (integer 1-5)
.RE
.TP
\fBDatebookDB\fR
.RS
.IP \fBcategory\fR 10
The category field of the appointment. (integer)
.IP \fBattribute\fR 10
The attribute field for the appointment. (integer)
.IP \fBevent\fR 10
(integer)
.IP \fBbegin\fR 10
(\fBclock\fR format time)
.IP \fBend\fR 10
(\fBclock\fR format time)
.IP \fBrepeatType\fR 10
(integer)
.IP \fBrepeatForever\fR 10
(integer 0|1)
.IP \fBrepeatEnd\fR 10
(\fBclock\fR format time when repeatForever is false)
.IP \fBrepeatFreq\fR 10
(integer)
.IP \fBrepeatOn\fR 10
(integer)
.IP \fBrepeatWeekstart\fR 10
(integer)
.IP \fBexceptions\fR 10
(list of \fBclock\fB format time)
.IP \fBdescription\fR 10
The text description of the appointment (string)
.IP \fBnote\fR 10
An optional note for the appointement (string)
.RE
.SH BUGS
.PP
Several of the routines are untested.

.SH SEE ALSO
\fBlibpui(3)\fR

.SH AUTHOR
Scotter <scotter@iname.com>

.SH COPYRIGHT
Copyright (C) 1997 Scotter <scotter@iname.com>
.PP
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
.PP
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
.PP
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
