/* 
 *
 * pdb2pui.c
 *
 * Convert a pilot ".pdb" file into a ".pui" database for modification.
 *
 * Copyright (c) 1996, Scotter <scotter@iname.com>
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file LICENSE for details.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pi-file.h"
#include "pui.h"

main(int argc, char *argv[])
{
  struct pi_file *pf;
  struct DBInfo info;
  PUI_FILE puidb;
  void *buffer;
  int num_records;
  int i;

  if (argc != 3) {
    fprintf(stderr,"usage:%s pdbfile.pdb puifile.pui\n",argv[0]);
    fprintf(stderr, "Converts a pilot database file into UNIX gdbm database.\n");
    exit(2);
  }

  pf = pi_file_open(argv[1]);
  if (pf == 0) {
    fprintf(stderr, "Error: Couldn't open %s for reading.\n", argv[1]);
    exit(1);
  }

  pi_file_get_info(pf, &info);
  printf("Database is: %s\n", info.name);
  if (info.flags & dlpDBFlagResource) {
    fprintf(stderr, "Error: source is a resource file, not a database file.\n");
    pi_file_close(pf);
    exit(1);
  }

  puidb = pui_Create(argv[2], &info);
  if (puidb == 0) {
    fprintf(stderr, "Error: Couldn't open %s for writing.\n", argv[2]);
    pi_file_close(pf);
    exit(1);
  }

  pi_file_get_app_info(pf, &buffer, &i);
  if (i>0) {
    pui_WriteAppInfo(puidb, i, buffer);
  }

  pi_file_get_entries(pf, &num_records);

  for (i=0;i<num_records;i++) {
    int len;
    int size;
    int attr;
    int category;
    unsigned long id;

    len = pi_file_read_record(pf, i, &buffer, &size, &attr, &category, &id);
    if(len<0) /* Safety */
      break;

    /* Insert "0" appends to list */
    pui_Insert(puidb, 0, id, attr, category, size, (char *)buffer);
  }

  pui_Close(puidb);

  pi_file_close(pf);

  exit(0);
}
