/*
 * pui-clean.c
 *
 * Remove deleted (including archived) records from a database.
 *
 * Copyright (c) 1996, Scotter <scotter@iname.com>
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file LICENSE for details.
 *
 */
#include <stdio.h>
#include "pui.h"

main(int argc, char *argv[])
{
  PUI_FILE db;
  PUI_FILE arc = 0;
  unsigned long id;

  if (argc < 2 || argc > 3) {
    goto usage;
  }
  db = pui_Open(argv[1], "w");
  if (!db) {
    fprintf(stderr, "Error: trouble opening %s for writing.\n", argv[1]);
    goto usage;
  }
  if (argc == 3) {
    arc = pui_Open(argv[2], "w");
    if (!arc) {
      fprintf(stderr, "Error: trouble opening %s for writing.\n", argv[2]);
      goto usage;
    }
  }

  id = pui_First(db);
  printf("Deleting tagged records"); fflush(stdout);
  while(id) {
    int attr, category;
    unsigned long nextid;
    pui_ReadHeader(db, id, &attr, &category);
    nextid = pui_Next(db, id);
    /* If Record is tagged for deletion or archive... */
    if (attr & 0x80 || attr & 0x08) {
      if (arc && (attr & 0x08)) { /* Record is tagged for archive */
        int size;
        char buffer[0xffff];
        pui_ReadRecord(db, id, &size, buffer);
        pui_Insert(arc, 0, 0, attr&0x77, category, size, buffer);
        printf("+"); fflush(stdout);
      } else {
        printf("."); fflush(stdout);
      }
      pui_Delete(db, id);
    }
    id = nextid;
  }

  if (arc)
    pui_Close(arc);

  printf(" Done.\n"); fflush(stdout);
  printf("Reorganizing database..."); fflush(stdout);
  pui_Reorganize(db);
  printf("Done.\n"); fflush(stdout);

  pui_Close(db);

  exit(0);
 
  usage:
    fprintf(stderr, "%s <puidb> [<archivedb>]\n", argv[0]);
    fprintf(stderr, "Remove records from the puidb that have been tagged\n");
    fprintf(stderr, "for deletion. If a second database is provided, records\n");
    fprintf(stderr, "that have been tagged for archive will be added to it.\n");
    exit(-1);
}
