/*
 * pui-dump.c
 *
 * Copyright (c) 1996, Scotter <scotter@iname.com>
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file LICENSE for details.
 *
 */
#include <stdio.h>
#include "pui.h"
#include "pui_io.h"

/* Stolen from pilot-link utils.c */
dumpline (char *buf, int len, int addr)
{
  int i;

  fprintf (stderr,"%.4x  ",addr);

  for (i=0;i<16;i++) {

    if (i<len) fprintf (stderr,"%.2x ",0xff & (unsigned int)buf[i]);
    else fprintf (stderr,"   ");
  }

  fprintf (stderr,"  ");

  for (i=0;i<len;i++) {
    if (isprint(buf[i])) fprintf (stderr,"%c",buf[i]);
    else fprintf(stderr,".");
  }
  fprintf(stderr,"\n");
}

/* Stolen from utils.c */
dumpdata (char * buf, int len) {
  int i;
  for(i=0;i<len;i+=16) {
    dumpline(buf+i, ((len-i)>16) ? 16 : len-i, i);
  }
}

main(int argc, char *argv[])
{
  PUI_FILE db;
  unsigned long id;

  char *version;
  char *name;
  struct pui_global_record global;
  struct pui_header_record header;
  struct pui_data_record   data;
  struct DBInfo info;
  int i;

  char buffer[0xffff]; /* For data record. */

  db = pui_Open(argv[1], "r");

  if (!db) {
    exit(-1);
  }

  fprintf(stderr, "Pui open returns: %lx\n\n", db);
    
  version = pui_Version(db);
  fprintf(stderr, "Database file version: %s\n\n", version);

  pui_ReadDBInfo(db, &info);
  fprintf(stderr, "DBInfo:\n");
  fprintf(stderr, "        more: %d\n", info.more);
  fprintf(stderr, "       flags: %x\n", info.flags);
  fprintf(stderr, "        type: %lx\n", info.type);
  fprintf(stderr, "     creator: %lx\n", info.creator);
  fprintf(stderr, "     version: %d\n", info.version);
  fprintf(stderr, "      modnum: %lx\n", info.modnum);
  fprintf(stderr, "      crdate: %s", ctime(&info.crdate));
  fprintf(stderr, "     moddate: %s", ctime(&info.moddate));
  fprintf(stderr, "  backupdate: %s", ctime(&info.backupdate));
  fprintf(stderr, "       index: %d\n", info.index);
  fprintf(stderr, "        name: %s\n\n", info.name);

  pui_ReadAppInfo(db, &i, buffer);
  if (i <= 0) {
    fprintf(stderr, "No AppInfo.\n");
  } else {
    fprintf(stderr, "AppInfo:\n");
    fprintf(stderr, "  Size: %d\n", i);
    fprintf(stderr, "  Data:\n");
    dumpdata(buffer, i);
  }
  fprintf(stderr, "\n");

  pui_global_read(db, &global);
  fprintf(stderr, "Global data:\n");
  fprintf(stderr, "  num_records: %d\n", global.num_records);
  fprintf(stderr, "         head: %lx\n", global.head);
  fprintf(stderr, "         tail: %lx\n\n", global.tail);

  data.length = 0;
  data.data = buffer; /* Provide an output buffer. */

  id = pui_First(db);
  while(id) {
    pui_header_read(db, id, &header);
    pui_record_read(db, id, &data);
    fprintf(stderr, "Record: %lx\n", id);
    fprintf(stderr, "  Header:\n");
    fprintf(stderr, "        attr: %d\n", header.attr);
    fprintf(stderr, "    category: %d\n", header.category);
    fprintf(stderr, "        prev: %lx\n", header.prev);
    fprintf(stderr, "        next: %lx\n", header.next);
    fprintf(stderr, "  Data:\n");
    fprintf(stderr, "      length: %d\n", data.length);
    dumpdata(data.data, data.length);
    fprintf(stderr, "\n");
    id = pui_Next(db, id);
  }

  pui_Close(db);
}
