#ifndef PUI_H
#define PUI_H

#include "pi-dlp.h"

/*
 * pui.h
 *
 * These are the "public" routines to pui.
 *
 * Copyright (c) 1996, Scotter <scotter@iname.com>
 *
 * This is free software, licensed under the GNU Public License V2.
 *
 */

typedef unsigned long PUI_FILE; /* Hide GDBM_FILE type. */

PUI_FILE pui_Create(char *filename, struct DBInfo *info);
PUI_FILE pui_Open (char *filename, const char *mode);
void     pui_Close(const PUI_FILE db);

char    *pui_Name(PUI_FILE db);
char    *pui_Version(PUI_FILE db);
void     pui_Reorganize(const PUI_FILE db);

unsigned long pui_First(PUI_FILE db);
unsigned long pui_Last(PUI_FILE db);
unsigned long pui_Prev(PUI_FILE db, unsigned long id);
unsigned long pui_Next(PUI_FILE db, unsigned long id);
unsigned long pui_NumRecords(PUI_FILE db);
unsigned long pui_GetByIndex(PUI_FILE db, int i);

int           pui_Exists(PUI_FILE db, unsigned long id);
unsigned long pui_UniqueId(PUI_FILE db);

void pui_ReadDBInfo (PUI_FILE db, struct DBInfo *info);
void pui_WriteDBInfo(PUI_FILE db, struct DBInfo *info);

void pui_ReadAppInfo (PUI_FILE db, int *size, char *buffer);
void pui_WriteAppInfo(PUI_FILE db, int  size, char *buffer);

void pui_ReadHeader(PUI_FILE db, unsigned long id, int *attr, int *category);
void pui_ReadRecord(PUI_FILE db, unsigned long id, int *size, char *buffer);
void pui_Read      (PUI_FILE db, unsigned long id, int *attr, int *category,
                                                   int *size, char *buffer);

void pui_WriteHeader(PUI_FILE db, unsigned long id, int attr, int category);
void pui_WriteRecord(PUI_FILE db, unsigned long id, int size, char *buffer);
void pui_Write      (PUI_FILE db, unsigned long id, int attr, int category,
                                                    int size, char *buffer);

unsigned long pui_Insert(PUI_FILE db,
                unsigned long nextid,
                unsigned long id,
                int attr,
                int category,
                int size,
                char *buffer);

unsigned long pui_InsertAtIndex(PUI_FILE db,
                       int i,
                       unsigned long id,
                       int attr,
                       int category,
                       int size,
                       char *buffer);

void pui_Delete(PUI_FILE db, unsigned long id);
void pui_DeleteByIndex(PUI_FILE db, int i);

#endif
