/* 
 *
 * pui2pdb.c
 *
 * Convert a ".pui" database file into a pilot ".pdb" file.
 *
 * Copyright (c) 1996, Scotter <scotter@iname.com>
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file LICENSE for details.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pi-file.h"
#include "pui.h"

main(int argc, char *argv[])
{
  struct pi_file *pf;
  struct DBInfo info;
  PUI_FILE puidb;
  char buffer[0xffff];
  int i;
  unsigned long id;

  if (argc != 3) {
    fprintf(stderr,"usage:%s puifile.pui pdbfile.pdb\n",argv[0]);
    fprintf(stderr, "Converts a UNIX gdbm database into a pilot database file.\n");
    exit(2);
  }

  puidb = pui_Open(argv[1], "r");
  if (puidb == 0) {
    fprintf(stderr, "Error: Couldn't open %s for reading.\n", argv[1]);
    exit(1);
  }

  pui_ReadDBInfo(puidb, &info);
  printf("Database is: %s\n", info.name);

  pf = pi_file_create(argv[2], &info);
  if (pf == 0) {
    fprintf(stderr, "Error: Couldn't open %s for writing.\n", argv[2]);
    pui_Close(puidb);
    exit(1);
  }

  pui_ReadAppInfo(puidb, &i, buffer);
  if (i>0) {
    pi_file_set_app_info(pf, buffer, i);
  }

  id = pui_First(puidb);
  while(id) {
    int attr;
    int category;
    int size;
    pui_Read(puidb, id, &attr, &category, &size, buffer);
    if (pi_file_append_record(pf, buffer, size, attr, category, id)<0) {
      printf("Error: trouble appending record to pdb file.\n");
      pui_Close(puidb);
      return -1;
    }
    id = pui_Next(puidb, id);
  }

  pui_Close(puidb);

  pi_file_close(pf);

  exit(0);
}
