/*
 * tclDatebookDB.c
 *
 * Copyright (c) 1996, Scotter <scotter@iname.com>
 *
 * This is free software, licensed under the GNU Public License V2.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tcl.h"
#include "pi-datebook.h"
#include "pui.h"
#include "tclpui.h"

static void tclDatebookDB_NewDBInfo(struct DBInfo *);
static void tclDatebookDB_Read(Tcl_Interp *, char *, int , char *);
static void tclDatebookDB_Write(Tcl_Interp *, char *, int *, char *);
static void tclDatebookDB_AppInfoRead(Tcl_Interp *, char *, PUI_FILE);
static void tclDatebookDB_AppInfoWrite(Tcl_Interp *, char *, PUI_FILE);

/*
 * Define the public symbol "object" through which we access the routines
 * in this module.
 */
struct tclDBModule tclDatebookDB = {
  "DatebookDB", tclDatebookDB_NewDBInfo,
                tclDatebookDB_Read,
                tclDatebookDB_Write,
                tclDatebookDB_AppInfoRead,
                tclDatebookDB_AppInfoWrite
};

/*---------------------DatebookDB-----------------------*/

static void tclDatebookDB_NewDBInfo(struct DBInfo *info)
{
  info->type = 0x44415441;
  info->creator = 0x64617465;
  info->crdate = 855693237;
  info->moddate = 855693237;
  info->backupdate = -2082816000;
  info->modnum = 6;
  info->flags = 0;
  info->version = 0;
  strncpy(info->name, "DatebookDB", 32);
  info->index = 0;
  info->more = 0;
}

static void tclDatebookDB_Read(Tcl_Interp *interp, char *varname, int size, char *buffer)
{
  char s[1024];
  struct Appointment apt;
  time_t t;
  int i;

  unpack_Appointment(&apt, buffer, size);

  sprintf(s, "%d", apt.event);
  Tcl_SetVar2(interp, varname, "event", s, 0);

  t = mktime(&apt.begin);
  sprintf(s, "%ld", t);
  Tcl_SetVar2(interp, varname, "begin", s, 0);

  t = mktime(&apt.end);
  sprintf(s, "%ld", t);
  Tcl_SetVar2(interp, varname, "end", s, 0);

  sprintf(s, "%d", apt.advance);
  Tcl_SetVar2(interp, varname, "advance", s, 0);
  sprintf(s, "%d", apt.advanceUnits);
  Tcl_SetVar2(interp, varname, "advanceUnits", s, 0);

  sprintf(s, "%d", apt.repeatType);
  Tcl_SetVar2(interp, varname, "repeatType", s, 0);

  if (apt.repeatForever) {
    Tcl_SetVar2(interp, varname, "repeatEnd", "", 0);
  } else {
    t = mktime(&apt.repeatEnd);
    sprintf(s, "%ld", t);
    Tcl_SetVar2(interp, varname, "repeatEnd", s, 0);
  }

  sprintf(s, "%d", apt.repeatForever);
  Tcl_SetVar2(interp, varname, "repeatForever", s, 0);
  sprintf(s, "%d", apt.repeatFreq);
  Tcl_SetVar2(interp, varname, "repeatFreq", s, 0);
  sprintf(s, "%d", apt.repeatOn);
  Tcl_SetVar2(interp, varname, "repeatOn", s, 0);
  sprintf(s, "%d", apt.repeatWeekstart);
  Tcl_SetVar2(interp, varname, "repeatWeekstart", s, 0);

  /* The exception count/list of exceptions is ONE element */
  /* containing a tcl list of the exceptions. */
  sprintf(s, "");
  for (i = 0; i < apt.exceptions; ++i) {
    t = mktime(&apt.exception[i]);
    sprintf(s, "%s %ld", s, t);
  }
  Tcl_SetVar2(interp, varname, "exceptions", s, 0);

  Tcl_SetVar2(interp, varname, "description", SAFE(apt.description), 0);
  Tcl_SetVar2(interp, varname, "note", SAFE(apt.note), 0);

  free_Appointment(&apt);
}

static void tclDatebookDB_Write(Tcl_Interp *interp, char *varname, int *size, char *buffer)
{
  struct Appointment apt;
  char *value;
  time_t secs;
  struct tm *t;
  int myargc = 0;
  char **myargv = 0;
  int i;

  value = Tcl_GetVar2(interp, varname, "event", 0);
  apt.event = SAFEATOI(value);

  value = Tcl_GetVar2(interp, varname, "begin", 0);
  secs = (time_t)SAFESTRTOUL(value, 10);
  t = localtime(&secs);
  memcpy(&apt.begin, t, sizeof(struct tm));

  value = Tcl_GetVar2(interp, varname, "end", 0);
  secs = (time_t)SAFESTRTOUL(value, 10);
  t = localtime(&secs);
  memcpy(&apt.end, t, sizeof(struct tm));

  value = Tcl_GetVar2(interp, varname, "advance", 0);
  apt.advance = SAFEATOI(value);

  value = Tcl_GetVar2(interp, varname, "advanceUnits", 0);
  apt.advanceUnits = SAFEATOI(value);

  value = Tcl_GetVar2(interp, varname, "repeatType", 0);
  apt.repeatType = SAFEATOI(value);

  value = Tcl_GetVar2(interp, varname, "repeatForever", 0);
  apt.repeatForever = SAFEATOI(value);

  /* Convert for use with tcl "clock" format */
  if (apt.repeatForever == 0) {
    value = Tcl_GetVar2(interp, varname, "repeatEnd", 0);
    secs = (time_t)SAFESTRTOUL(value, 10);
    t = localtime(&secs);
    memcpy(&apt.repeatEnd, t, sizeof(struct tm));
  }

  value = Tcl_GetVar2(interp, varname, "repeatFreq", 0);
  apt.repeatFreq = SAFEATOI(value);

  value = Tcl_GetVar2(interp, varname, "repeatOn", 0);
  apt.repeatOn = SAFEATOI(value);

  value = Tcl_GetVar2(interp, varname, "repeatWeekstart", 0);
  apt.repeatWeekstart = SAFEATOI(value);

  value = Tcl_GetVar2(interp, varname, "repeatExceptions", 0);
  apt.exceptions = SAFEATOI(value);

  /* Parse exceptions list */
  value = Tcl_GetVar2(interp, varname, "exceptions", 0);
  Tcl_SplitList(interp, value, &myargc, &myargv); /* !!!! Check for error */
  apt.exceptions = myargc;
  if (apt.exceptions) {
    apt.exception = (struct tm *)malloc(sizeof(struct tm)*apt.exceptions);
    for (i = 0; i < apt.exceptions; ++i) {
      secs = (time_t)SAFESTRTOUL(myargv[i], 10);
      t = localtime(&secs);
      memcpy(&apt.exception[i], t, sizeof(struct tm));
    }
  } else {
    apt.exception = 0;
  }
  if (myargv) free(myargv);

  value = Tcl_GetVar2(interp, varname, "description", 0);
  apt.description = value?(*value?strdup(value):0):0;
  value = Tcl_GetVar2(interp, varname, "note", 0);
  apt.note = value?(*value?strdup(value):0):0;

  pack_Appointment(&apt, buffer, size);

  free_Appointment(&apt);
}

static void tclDatebookDB_AppInfoRead(Tcl_Interp *interp, char *varname, PUI_FILE puidb)
{
  char s[32];
  struct AppointmentAppInfo appinfo;
  int i;
  char buffer[0xffff];
  int size;

  pui_ReadAppInfo(puidb, &size, buffer);

  unpack_AppointmentAppInfo(&appinfo, buffer, size);

  sprintf(s, "%u", appinfo.renamedcategories);
  Tcl_SetVar2(interp, varname, "renamedcategories", s, 0);

  for (i=0;i<16;++i) {
    sprintf(s, "CategoryName,%d", i);
    Tcl_SetVar2(interp, varname, s, appinfo.CategoryName[i], 0);
  }
  Tcl_SetVar2(interp, varname, "CategoryID", appinfo.CategoryID, 0);

  sprintf(s, "%u", appinfo.lastUniqueID);
  Tcl_SetVar2(interp, varname, "lastUniqueID", s, 0);

  sprintf(s, "%d", appinfo.startOfWeek);
  Tcl_SetVar2(interp, varname, "startOfWeek", s, 0);
}

static void tclDatebookDB_AppInfoWrite(Tcl_Interp *interp, char *varname, PUI_FILE puidb)
{
  char s[32];
  struct AppointmentAppInfo appinfo;
  char *value;
  int i;
  char buffer[0xffff];
  int size;

  value = Tcl_GetVar2(interp, varname, "renamedcategories", 0);
  appinfo.renamedcategories = SAFESTRTOUL(value,10);

  for (i=0;i<16;++i) {
    sprintf(s, "CategoryName,%d", i);
    value = Tcl_GetVar2(interp, varname, s, 0);
    SAFESTRINGCOPY(appinfo.CategoryName[i],value);
  }

  value = Tcl_GetVar2(interp, varname, "CategoryID", 0);
  SAFESTRINGCOPY(appinfo.CategoryID,value);

  value = Tcl_GetVar2(interp, varname, "lastUniqueID", 0);
  appinfo.lastUniqueID = (unsigned char)SAFESTRTOUL(value,10);

  value = Tcl_GetVar2(interp, varname, "startOfWeek", 0);
  appinfo.startOfWeek = SAFEATOI(value);
  
  pack_AppointmentAppInfo(&appinfo, buffer, &size);
  pui_WriteAppInfo(puidb, size, buffer);
}
