/*
 * tclMemoDB.c
 *
 * Copyright (c) 1996, Scotter <scotter@iname.com>
 *
 * This is free software, licensed under the GNU Public License V2.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tcl.h"
#include "pi-memo.h"
#include "pui.h"
#include "tclpui.h"

static void tclMemoDB_NewDBInfo(struct DBInfo *);
static void tclMemoDB_Read(Tcl_Interp *, char *, int , char *);
static void tclMemoDB_Write(Tcl_Interp *, char *, int *, char *);
static void tclMemoDB_AppInfoRead(Tcl_Interp *, char *, PUI_FILE);
static void tclMemoDB_AppInfoWrite(Tcl_Interp *, char *, PUI_FILE);

/*
 * Define the public symbol "object" through which we access the routines
 * in this module.
 */
struct tclDBModule tclMemoDB = {
  "MemoDB", tclMemoDB_NewDBInfo,
            tclMemoDB_Read,
            tclMemoDB_Write,
            tclMemoDB_AppInfoRead,
            tclMemoDB_AppInfoWrite
};
 

/*---------------------MemoDB-----------------------*/

static void tclMemoDB_NewDBInfo(struct DBInfo *info)
{
  info->type = 0x44415441;
  info->creator = 0x6d656d6f;
  info->crdate = 818137284;
  info->moddate = 818137284;
  info->backupdate = -2082816000;
  info->modnum = 1;
  info->flags = 0;
  info->version = 0;
  strncpy(info->name, "MemoDB", 32);
  info->index = 0;
  info->more = 0;
}

static void tclMemoDB_Read(Tcl_Interp *interp, char *varname, int size, char *buffer)
{
  struct Memo memo;

  unpack_Memo(&memo, buffer, size);
  Tcl_SetVar2(interp, varname, "text", memo.text, 0);

  free_Memo(&memo);
}

static void tclMemoDB_Write(Tcl_Interp *interp, char *varname, int *size, char *buffer)
{
  struct Memo memo;
  char *value;

  value = Tcl_GetVar2(interp, varname, "text", 0);
  memo.text = value?(*value?strdup(value):0):0;
  pack_Memo(&memo, buffer, size);

  free_Memo(&memo);
}

static void tclMemoDB_AppInfoRead(Tcl_Interp *interp, char *varname, PUI_FILE puidb)
{
  char s[32];
  struct MemoAppInfo appinfo;
  int i;
  char buffer[0xffff];
  int size;

  pui_ReadAppInfo(puidb, &size, buffer);

  unpack_MemoAppInfo(&appinfo, buffer, size);

  sprintf(s, "%u", appinfo.renamedcategories);
  Tcl_SetVar2(interp, varname, "renamedcategories", s, 0);

  for (i=0;i<16;++i) {
    sprintf(s, "CategoryName,%d", i);
    Tcl_SetVar2(interp, varname, s, appinfo.CategoryName[i], 0);
  }
  Tcl_SetVar2(interp, varname, "CategoryID", appinfo.CategoryID, 0);

  sprintf(s, "%u", appinfo.lastUniqueID);
  Tcl_SetVar2(interp, varname, "lastUniqueID", s, 0);
}

static void tclMemoDB_AppInfoWrite(Tcl_Interp *interp, char *varname, PUI_FILE puidb)
{
  char s[32];
  struct MemoAppInfo appinfo;
  char *value;
  int i;
  char buffer[0xffff];
  int size;

  value = Tcl_GetVar2(interp, varname, "renamedcategories", 0);
  appinfo.renamedcategories = SAFESTRTOUL(value,10);

  for (i=0;i<16;++i) {
    sprintf(s, "CategoryName,%d", i);
    value = Tcl_GetVar2(interp, varname, s, 0);
    SAFESTRINGCOPY(appinfo.CategoryName[i],value);
  }

  value = Tcl_GetVar2(interp, varname, "CategoryID", 0);
  SAFESTRINGCOPY(appinfo.CategoryID,value);

  value = Tcl_GetVar2(interp, varname, "lastUniqueID", 0);
  appinfo.lastUniqueID = (unsigned char)SAFESTRTOUL(value,10);

  pack_MemoAppInfo(&appinfo, buffer, &size);
  pui_WriteAppInfo(puidb, size, buffer);
}
